@@ -19,10 +19,10 @@ Routines for configuring Glance
 
 import logging
 import os
-import sys
 
 from oslo_config import cfg
 from oslo_middleware import cors
+from oslo_policy import opts
 from oslo_policy import policy
 from paste import deploy
 
@@ -36,7 +36,7 @@ paste_deploy_opts = [
 Deployment flavor to use in the server application pipeline.
 
 Provide a string value representing the appropriate deployment
-flavor used in the server application pipleline. This is typically
+flavor used in the server application pipeline. This is typically
 the partial name of a pipeline in the paste configuration file with
 the service name removed.
 
@@ -57,7 +57,7 @@ Related Options:
 Name of the paste configuration file.
 
 Provide a string value representing the name of the paste
-configuration file to use for configuring piplelines for
+configuration file to use for configuring pipelines for
 server application deployments.
 
 NOTES:
@@ -99,6 +99,18 @@ image_format_opts = [
                        "image attribute"),
                 deprecated_opts=[cfg.DeprecatedOpt('disk_formats',
                                                    group='DEFAULT')]),
+    cfg.ListOpt('vmdk_allowed_types',
+                default=['streamOptimized', 'monolithicSparse'],
+                help=_("A list of strings describing allowed VMDK "
+                       "'create-type' subformats that will be allowed. "
+                       "This is recommended to only include "
+                       "single-file-with-sparse-header variants to avoid "
+                       "potential host file exposure due to processing named "
+                       "extents. If this list is empty, then no VDMK image "
+                       "types allowed. Note that this is currently only "
+                       "checked during image conversion (if enabled), and "
+                       "limits the types of VMDK images we will convert "
+                       "from.")),
 ]
 task_opts = [
     cfg.IntOpt('task_time_to_live',
@@ -349,7 +361,7 @@ Related options:
     * location_strategy
 
 """)),
-    # NOTE(flaper87): The policy.json file should be updated and the locaiton
+    # NOTE(flaper87): The policy.yaml file should be updated and the location
     # related rules set to admin only once this option is finally removed.
     # NOTE(rosmaita): Unfortunately, this option is used to gate some code
     # paths; if the location related policies are set admin-only, then no
@@ -440,14 +452,29 @@ TeraBytes respectively. Note that there should not be any space between the
 value and unit. Value ``0`` signifies no quota enforcement. Negative values
 are invalid and result in errors.
 
+This has no effect if ``use_keystone_limits`` is enabled.
+
 Possible values:
     * A string that is a valid concatenation of a non-negative integer
       representing the storage value and an optional string literal
       representing storage units as mentioned above.
 
 Related options:
-    * None
+    * use_keystone_limits
 
+""")),
+    cfg.BoolOpt('use_keystone_limits', default=False,
+                help=_("""
+Utilize per-tenant resource limits registered in Keystone.
+
+Enabling this feature will cause Glance to retrieve limits set in keystone
+for resource consumption and enforce them against API users. Before turning
+this on, the limits need to be registered in Keystone or all quotas will be
+considered to be zero, and thus reject all new resource requests.
+
+These per-tenant resource limits are independent from the static
+global ones configured in this config file. If this is enabled, the
+relevant static global limits will be ignored.
 """)),
     cfg.HostAddressOpt('pydev_worker_debug_host',
                        sample_default='localhost',
@@ -562,9 +589,55 @@ Related options:
     List of enabled Image Import Methods
 
     'glance-direct', 'copy-image' and 'web-download' are enabled by default.
+    'glance-download' is available, but requires federated deployments.
 
     Related options:
         * [DEFAULT]/node_staging_uri""")),
+    cfg.BoolOpt('enforce_secure_rbac', default=False,
+                deprecated_for_removal=True,
+                deprecated_reason=_("""
+This option has been introduced to require operators to opt into enforcing
+authorization based on common RBAC personas, which is EXPERIMENTAL as of the
+Wallaby release. This behavior will be the default and STABLE in a future
+release, allowing this option to be removed.
+"""),
+                deprecated_since='Wallaby',
+                help=_("""
+Enforce API access based on common persona definitions used across OpenStack.
+Enabling this option formalizes project-specific read/write operations, like
+creating private images or updating the status of shared image, behind the
+`member` role. It also formalizes a read-only variant useful for
+project-specific API operations, like listing private images in a project,
+behind the `reader` role.
+
+Operators should take an opportunity to understand glance's new image policies,
+audit assignments in their deployment, and update permissions using the default
+roles in keystone (e.g., `admin`, `member`, and `reader`).
+
+Related options:
+    * [oslo_policy]/enforce_new_defaults
+""")),
+    cfg.StrOpt('worker_self_reference_url',
+               default=None,
+               help=_("""
+The URL to this worker.
+
+If this is set, other glance workers will know how to contact this one
+directly if needed. For image import, a single worker stages the image
+and other workers need to be able to proxy the import request to the
+right one.
+
+If unset, this will be considered to be `public_endpoint`, which
+normally would be set to the same value on all workers, effectively
+disabling the proxying behavior.
+
+Possible values:
+    * A URL by which this worker is reachable from other workers
+
+Related options:
+    * public_endpoint
+
+""")),
 ]
 
 wsgi_opts = [
@@ -580,13 +653,13 @@ may overwhelm other system resources such as disk or outbound network
 bandwidth. If this is too small, image import requests will have to wait
 until a thread becomes available to begin processing.""")),
     cfg.StrOpt('python_interpreter',
-               default=sys.executable,
+               default=None,
                help=_("""
 Path to the python interpreter to use when spawning external
-processes. By default this is sys.executable, which should be the
-same interpreter running Glance itself. However, in some situations
-(i.e. uwsgi) this may not actually point to a python interpreter
-itself.""")),
+processes. If left unspecified, this will be sys.executable, which should
+be the same interpreter running Glance itself. However, in some situations
+(for example, uwsgi) sys.executable may not actually point to a python
+interpreter and an alternative value must be set.""")),
 ]
 
 
@@ -698,6 +771,12 @@ def set_config_defaults():
     """This method updates all configuration default values."""
     set_cors_middleware_defaults()
 
+    # TODO(gmann): Remove setting the default value of config policy_file
+    # once oslo_policy change the default value to 'policy.yaml'.
+    # https://github.com/openstack/oslo.policy/blob/a626ad12fe5a3abd49d70e3e5b95589d279ab578/oslo_policy/opts.py#L49
+    DEFAULT_POLICY_FILE = 'policy.yaml'
+    opts.set_defaults(cfg.CONF, DEFAULT_POLICY_FILE)
+
 
 def set_cors_middleware_defaults():
     """Update default configuration options for oslo.middleware."""
