@@ -31,22 +31,20 @@ from eventlet.green import socket
 import functools
 import os
 import re
-import uuid
+import urllib
 
-from OpenSSL import crypto
+import glance_store
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_utils import encodeutils
 from oslo_utils import excutils
 from oslo_utils import netutils
 from oslo_utils import strutils
-import six
-from six.moves import urllib
 from webob import exc
 
 from glance.common import exception
 from glance.common import timeutils
-from glance.i18n import _, _LE, _LW
+from glance.common import wsgi
+from glance.i18n import _, _LE
 
 CONF = cfg.CONF
 
@@ -128,7 +126,7 @@ def cooperative_read(fd):
 MAX_COOP_READER_BUFFER_SIZE = 134217728  # 128M seems like a sane buffer limit
 
 CONF.import_group('import_filtering_opts',
-                  'glance.async.flows._internal_plugins')
+                  'glance.async_.flows._internal_plugins')
 
 
 def validate_import_uri(uri):
@@ -218,7 +216,7 @@ class CooperativeReader(object):
                 result = self.buffer[self.position:]
                 self.buffer = b''
                 self.position = 0
-                return str(result)
+                return bytes(result)
             else:
                 # otherwise read the next chunk from the underlying iterator
                 # and return it as a whole. Reset the buffer, as subsequent
@@ -228,7 +226,7 @@ class CooperativeReader(object):
                         self.iterator = self.__iter__()
                     return next(self.iterator)
                 except StopIteration:
-                    return ''
+                    return b''
                 finally:
                     self.buffer = b''
                     self.position = 0
@@ -314,9 +312,9 @@ def image_meta_to_http_headers(image_meta):
                 for pk, pv in v.items():
                     if pv is not None:
                         headers["x-image-meta-property-%s"
-                                % pk.lower()] = six.text_type(pv)
+                                % pk.lower()] = str(pv)
             else:
-                headers["x-image-meta-%s" % k.lower()] = six.text_type(v)
+                headers["x-image-meta-%s" % k.lower()] = str(v)
     return headers
 
 
@@ -385,10 +383,9 @@ def create_mashup_dict(image_meta):
     """
 
     d = {}
-    for key, value in six.iteritems(image_meta):
+    for key, value in image_meta.items():
         if isinstance(value, dict):
-            for subkey, subvalue in six.iteritems(
-                    create_mashup_dict(value)):
+            for subkey, subvalue in create_mashup_dict(value).items():
                 if subkey not in image_meta:
                     d[subkey] = subvalue
         else:
@@ -440,68 +437,11 @@ def setup_remote_pydev_debug(host, port):
             LOG.exception(error_msg)
 
 
-def validate_key_cert(key_file, cert_file):
-    try:
-        error_key_name = "private key"
-        error_filename = key_file
-        with open(key_file, 'r') as keyfile:
-            key_str = keyfile.read()
-        key = crypto.load_privatekey(crypto.FILETYPE_PEM, key_str)
-
-        error_key_name = "certificate"
-        error_filename = cert_file
-        with open(cert_file, 'r') as certfile:
-            cert_str = certfile.read()
-        cert = crypto.load_certificate(crypto.FILETYPE_PEM, cert_str)
-    except IOError as ioe:
-        raise RuntimeError(_("There is a problem with your %(error_key_name)s "
-                             "%(error_filename)s.  Please verify it."
-                             "  Error: %(ioe)s") %
-                           {'error_key_name': error_key_name,
-                            'error_filename': error_filename,
-                            'ioe': ioe})
-    except crypto.Error as ce:
-        raise RuntimeError(_("There is a problem with your %(error_key_name)s "
-                             "%(error_filename)s.  Please verify it. OpenSSL"
-                             " error: %(ce)s") %
-                           {'error_key_name': error_key_name,
-                            'error_filename': error_filename,
-                            'ce': ce})
-
-    try:
-        data = str(uuid.uuid4())
-        # On Python 3, explicitly encode to UTF-8 to call crypto.sign() which
-        # requires bytes. Otherwise, it raises a deprecation warning (and
-        # will raise an error later).
-        data = encodeutils.to_utf8(data)
-        digest = CONF.digest_algorithm
-        if digest == 'sha1':
-            LOG.warn(
-                _LW('The FIPS (FEDERAL INFORMATION PROCESSING STANDARDS)'
-                    ' state that the SHA-1 is not suitable for'
-                    ' general-purpose digital signature applications (as'
-                    ' specified in FIPS 186-3) that require 112 bits of'
-                    ' security. The default value is sha1 in Kilo for a'
-                    ' smooth upgrade process, and it will be updated'
-                    ' with sha256 in next release(L).'))
-        out = crypto.sign(key, data, digest)
-        crypto.verify(cert, out, data, digest)
-    except crypto.Error as ce:
-        raise RuntimeError(_("There is a problem with your key pair.  "
-                             "Please verify that cert %(cert_file)s and "
-                             "key %(key_file)s belong together.  OpenSSL "
-                             "error %(ce)s") % {'cert_file': cert_file,
-                                                'key_file': key_file,
-                                                'ce': ce})
-
-
 def get_test_suite_socket():
     global GLANCE_TEST_SOCKET_FD_STR
     if GLANCE_TEST_SOCKET_FD_STR in os.environ:
         fd = int(os.environ[GLANCE_TEST_SOCKET_FD_STR])
         sock = socket.fromfd(fd, socket.AF_INET, socket.SOCK_STREAM)
-        if six.PY2:
-            sock = socket.SocketType(_sock=sock)
         sock.listen(CONF.backlog)
         del os.environ[GLANCE_TEST_SOCKET_FD_STR]
         os.close(fd)
@@ -516,7 +456,7 @@ def is_valid_hostname(hostname):
 
 def is_valid_fqdn(fqdn):
     """Verify whether a host is a valid FQDN."""
-    return re.match('^[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$', fqdn) is not None
+    return re.match(r'^[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$', fqdn) is not None
 
 
 def parse_valid_host_port(host_port):
@@ -559,10 +499,10 @@ def parse_valid_host_port(host_port):
 
 
 try:
-    REGEX_4BYTE_UNICODE = re.compile(u'[\U00010000-\U0010ffff]')
+    REGEX_4BYTE_UNICODE = re.compile('[\U00010000-\U0010ffff]')
 except re.error:
     # UCS-2 build case
-    REGEX_4BYTE_UNICODE = re.compile(u'[\uD800-\uDBFF][\uDC00-\uDFFF]')
+    REGEX_4BYTE_UNICODE = re.compile('[\uD800-\uDBFF][\uDC00-\uDFFF]')
 
 
 def no_4byte_params(f):
@@ -573,12 +513,14 @@ def no_4byte_params(f):
     def wrapper(*args, **kwargs):
 
         def _is_match(some_str):
-            return (isinstance(some_str, six.text_type) and
-                    REGEX_4BYTE_UNICODE.findall(some_str) != [])
+            return (
+                isinstance(some_str, str) and
+                REGEX_4BYTE_UNICODE.findall(some_str) != []
+            )
 
         def _check_dict(data_dict):
             # a dict of dicts has to be checked recursively
-            for key, value in six.iteritems(data_dict):
+            for key, value in data_dict.items():
                 if isinstance(value, dict):
                     _check_dict(value)
                 else:
@@ -613,10 +555,7 @@ def stash_conf_values():
     conf = {
         'bind_host': CONF.bind_host,
         'bind_port': CONF.bind_port,
-        'tcp_keepidle': CONF.cert_file,
         'backlog': CONF.backlog,
-        'key_file': CONF.key_file,
-        'cert_file': CONF.cert_file
     }
 
     return conf
@@ -726,3 +665,50 @@ def evaluate_filter_op(value, operator, threshold):
 
     msg = _("Unable to filter on a unknown operator.")
     raise exception.InvalidFilterOperatorValue(msg)
+
+
+def _get_available_stores():
+    available_stores = CONF.enabled_backends
+    stores = []
+    # Remove reserved stores from the available stores list
+    for store in available_stores:
+        # NOTE (abhishekk): http store is readonly and should be
+        # excluded from the list.
+        if available_stores[store] == 'http':
+            continue
+        if store not in wsgi.RESERVED_STORES:
+            stores.append(store)
+
+    return stores
+
+
+def get_stores_from_request(req, body):
+    """Processes a supplied request and extract stores from it
+
+    :param req: request to process
+    :param body: request body
+
+    :raises glance_store.UnknownScheme:  if a store is not valid
+    :return: a list of stores
+    """
+    if body.get('all_stores', False):
+        if 'stores' in body or 'x-image-meta-store' in req.headers:
+            msg = _("All_stores parameter can't be used with "
+                    "x-image-meta-store header or stores parameter")
+            raise exc.HTTPBadRequest(explanation=msg)
+        stores = _get_available_stores()
+    else:
+        try:
+            stores = body['stores']
+        except KeyError:
+            stores = [req.headers.get('x-image-meta-store',
+                                      CONF.glance_store.default_backend)]
+        else:
+            if 'x-image-meta-store' in req.headers:
+                msg = _("Stores parameter and x-image-meta-store header can't "
+                        "be both specified")
+                raise exc.HTTPBadRequest(explanation=msg)
+    # Validate each store
+    for store in stores:
+        glance_store.get_store_from_store_identifier(store)
+    return stores
