@@ -29,7 +29,6 @@ import signal
 import struct
 import subprocess
 import sys
-import threading
 import time
 
 from eventlet.green import socket
@@ -45,7 +44,6 @@ from oslo_utils import encodeutils
 from oslo_utils import strutils
 from osprofiler import opts as profiler_opts
 import routes.middleware
-import six
 import webob.dec
 import webob.exc
 from webob import multidict
@@ -54,6 +52,8 @@ from glance.common import config
 from glance.common import exception
 from glance.common import store_utils
 from glance.common import utils
+import glance.db
+from glance import housekeeping
 from glance import i18n
 from glance.i18n import _, _LE, _LI, _LW
 
@@ -244,17 +244,6 @@ Related options:
 """)),
 ]
 
-wsgi_opts = [
-    cfg.StrOpt('secure_proxy_ssl_header',
-               deprecated_for_removal=True,
-               deprecated_reason=_('Use the http_proxy_to_wsgi middleware '
-                                   'instead.'),
-               help=_('The HTTP header used to determine the scheme for the '
-                      'original request, even if it was removed by an SSL '
-                      'terminating proxy. Typical value is '
-                      '"HTTP_X_FORWARDED_PROTO".')),
-]
-
 store_opts = [
     cfg.DictOpt('enabled_backends',
                 help=_('Key:Value pair of store identifier and store type. '
@@ -269,22 +258,6 @@ cli_opts = [
                     'used for inter-process communication.'),
 ]
 
-cache_opts = [
-    cfg.FloatOpt('cache_prefetcher_interval',
-                 default=300,
-                 help=_("""
-The interval in seconds to run periodic job cache_images.
-
-The cache_images method will fetch all images which are in queued state
-for caching in cache directory. The default value is 300.
-
-Possible values:
-    * Positive integer
-
-Related options:
-    * None
-"""))
-]
 
 LOG = logging.getLogger(__name__)
 
@@ -292,9 +265,7 @@ CONF = cfg.CONF
 CONF.register_opts(bind_opts)
 CONF.register_opts(socket_opts)
 CONF.register_opts(eventlet_opts)
-CONF.register_opts(wsgi_opts)
 CONF.register_opts(store_opts)
-CONF.register_opts(cache_opts)
 profiler_opts.set_defaults(CONF)
 
 ASYNC_EVENTLET_THREAD_POOL_LIST = []
@@ -427,15 +398,13 @@ def get_asynchronous_eventlet_pool(size=1000):
     return pool
 
 
-@six.add_metaclass(abc.ABCMeta)
-class BaseServer(object):
+class BaseServer(metaclass=abc.ABCMeta):
     """Server class to manage multiple WSGI sockets and applications.
 
     This class requires initialize_glance_store set to True if
     glance store needs to be initialized.
     """
-    def __init__(self, threads=1000, initialize_glance_store=False,
-                 initialize_prefetcher=False):
+    def __init__(self, threads=1000, initialize_glance_store=False):
         os.umask(0o27)  # ensure files are created with the correct privileges
         self._logger = logging.getLogger("eventlet.wsgi.server")
         self.threads = threads
@@ -445,21 +414,6 @@ class BaseServer(object):
         # NOTE(abhishek): Allows us to only re-initialize glance_store when
         # the API's configuration reloads.
         self.initialize_glance_store = initialize_glance_store
-        self.initialize_prefetcher = initialize_prefetcher
-        if self.initialize_prefetcher:
-            # NOTE(abhishekk): Importing the prefetcher just in time to avoid
-            # import loop during initialization
-            from glance.image_cache import prefetcher # noqa
-            self.prefetcher = prefetcher.Prefetcher()
-
-    def cache_images(self):
-        # After every 'cache_prefetcher_interval' this call will run and fetch
-        # all queued images into cache if there are any
-        cache_thread = threading.Timer(CONF.cache_prefetcher_interval,
-                                       self.cache_images)
-        cache_thread.daemon = True
-        cache_thread.start()
-        self.prefetcher.run()
 
     @staticmethod
     def set_signal_handler(signal_name, handler):
@@ -501,14 +455,15 @@ class BaseServer(object):
         self.default_port = default_port
         self.configure()
         self.start_wsgi()
-        if self.initialize_prefetcher:
-            self.cache_images()
+
+        cleaner = housekeeping.StagingStoreCleaner(glance.db.get_api())
+        self.pool.spawn_n(cleaner.clean_orphaned_staging_residue)
 
     def start_wsgi(self):
         workers = get_num_workers()
+        self.pool = self.create_pool()
         if workers == 0:
             # Useful for profiling, test, debug etc.
-            self.pool = self.create_pool()
             self.pool.spawn_n(self._single_run, self.application, self.sock)
             return
         else:
@@ -578,6 +533,14 @@ class BaseServer(object):
             for pool in ASYNC_EVENTLET_THREAD_POOL_LIST:
                 pool.waitall()
 
+        # NOTE(abhishekk): Importing the cache_images API module just
+        #  in time to avoid partial initialization of wsgi module
+        from glance.api.v2 import cached_images  # noqa
+        if cached_images.WORKER:
+            # If we started a cache worker, signal it to exit
+            # and wait until it does.
+            cached_images.WORKER.terminate()
+
     def _single_run(self, application, sock):
         """Start a WSGI server in a new green thread."""
         LOG.info(_LI("Starting single process server"))
@@ -669,14 +632,14 @@ class PosixServer(BaseServer):
             self.stale_children.remove(pid)
             LOG.info(_LI('Removed stale child %s'), pid)
         else:
-            LOG.warn(_LW('Unrecognised child %s') % pid)
+            LOG.warning(_LW('Unrecognised child %s'), pid)
 
     def _verify_and_respawn_children(self, pid, status):
         if len(self.stale_children) == 0:
             LOG.debug('No stale children')
         if os.WIFEXITED(status) and os.WEXITSTATUS(status) != 0:
             LOG.error(_LE('Not respawning child %d, cannot '
-                          'recover from termination') % pid)
+                          'recover from termination'), pid)
             if not self.children and not self.stale_children:
                 LOG.info(
                     _LI('All workers have terminated. Exiting'))
@@ -942,7 +905,7 @@ class Debug(Middleware):
         resp = req.get_response(self.application)
 
         print(("*" * 40) + " RESPONSE HEADERS")
-        for (key, value) in six.iteritems(resp.headers):
+        for key, value in resp.headers.items():
             print(key, "=", value)
         print('')
 
@@ -1080,10 +1043,6 @@ class Request(webob.Request):
     """Add some OpenStack API-specific logic to the base webob.Request."""
 
     def __init__(self, environ, *args, **kwargs):
-        if CONF.secure_proxy_ssl_header:
-            scheme = environ.get(CONF.secure_proxy_ssl_header)
-            if scheme:
-                environ['wsgi.url_scheme'] = scheme
         super(Request, self).__init__(environ, *args, **kwargs)
 
     @property
@@ -1099,26 +1058,6 @@ class Request(webob.Request):
         # not be inherited.
         webob.Request.body_file.fset(self, value)
 
-    @property
-    def params(self):
-        """Override params property of webob.request.BaseRequest.
-
-        Added an 'encoded_params' attribute in case of PY2 to avoid
-        encoding values in next subsequent calls to the params property.
-        """
-        if six.PY2:
-            encoded_params = getattr(self, 'encoded_params', None)
-            if encoded_params is None:
-                params = super(Request, self).params
-                params_dict = multidict.MultiDict()
-                for key, value in params.items():
-                    params_dict.add(key, encodeutils.safe_encode(value))
-
-                setattr(self, 'encoded_params',
-                        multidict.NestedMultiDict(params_dict))
-            return self.encoded_params
-        return super(Request, self).params
-
     def best_match_content_type(self):
         """Determine the requested response content-type."""
         supported = ('application/json',)
@@ -1148,7 +1087,7 @@ class Request(webob.Request):
         if not self.accept_language:
             return None
         langs = i18n.get_available_languages('glance')
-        # NOTE(rosmaita): give the webob lookup() function a sentinal value
+        # NOTE(rosmaita): give the webob lookup() function a sentinel value
         # for default so we can preserve the behavior of this function as
         # indicated by the current unit tests.  See Launchpad bug #1765748.
         best_match = self.accept_language.lookup(langs, default='fake_LANG')
@@ -1347,14 +1286,17 @@ class Resource(object):
             action_result = self.dispatch(self.controller, action,
                                           request, **action_args)
         except webob.exc.WSGIHTTPException as e:
-            exc_info = sys.exc_info()
             e = translate_exception(request, e)
-            six.reraise(type(e), e, exc_info[2])
+            raise e.with_traceback(sys.exc_info()[2])
         except UnicodeDecodeError:
             msg = _("Error decoding your request. Either the URL or the "
                     "request body contained characters that could not be "
                     "decoded by Glance")
             raise webob.exc.HTTPBadRequest(explanation=msg)
+        except exception.InvalidPropertyProtectionConfiguration as e:
+            LOG.exception(_LE("Caught error: %s"),
+                          encodeutils.exception_to_unicode(e))
+            raise webob.exc.HTTPBadRequest(explanation=e.msg)
         except Exception as e:
             LOG.exception(_LE("Caught error: %s"),
                           encodeutils.exception_to_unicode(e))
@@ -1368,10 +1310,6 @@ class Resource(object):
         try:
             response = webob.Response(request=request)
             self.dispatch(self.serializer, action, response, action_result)
-            # encode all headers in response to utf-8 to prevent unicode errors
-            for name, value in list(response.headers.items()):
-                if six.PY2 and isinstance(value, six.text_type):
-                    response.headers[name] = encodeutils.safe_encode(value)
             return response
         except webob.exc.WSGIHTTPException as e:
             return translate_exception(request, e)
