@@ -29,5 +29,5 @@ db_options.set_defaults(cfg.CONF)
 # Migration-related constants
 EXPAND_BRANCH = 'expand'
 CONTRACT_BRANCH = 'contract'
-CURRENT_RELEASE = 'yoga'
+CURRENT_RELEASE = 'zed'
 ALEMBIC_INIT_VERSION = 'liberty'
