@@ -98,16 +98,6 @@ def no_translate_debug_logs(logical_line, filename):
             yield(0, "G319: Don't translate debug level logs")
 
 
-@core.flake8ext
-def no_direct_use_of_unicode_function(logical_line):
-    """Check for use of unicode() builtin
-
-    G320
-    """
-    if unicode_func_re.match(logical_line):
-        yield(0, "G320: Use six.text_type() instead of unicode()")
-
-
 @core.flake8ext
 def check_no_contextlib_nested(logical_line):
     msg = ("G327: contextlib.nested is deprecated since Python 2.7. See "
@@ -127,31 +117,13 @@ def dict_constructor_with_list_copy(logical_line):
 
 
 @core.flake8ext
-def check_python3_xrange(logical_line):
-    if re.search(r"\bxrange\s*\(", logical_line):
-        yield(0, "G329: Do not use xrange. Use range, or six.moves.range for "
-                 "large loops.")
+def no_log_warn(logical_line):
+    """Disallow 'LOG.warn('
 
+    Use LOG.warning() instead of Deprecated LOG.warn().
+    https://docs.python.org/3/library/logging.html#logging.warning
+    """
 
-@core.flake8ext
-def check_python3_no_iteritems(logical_line):
-    msg = ("G330: Use six.iteritems() or dict.items() instead of "
-           "dict.iteritems().")
-    if re.search(r".*\.iteritems\(\)", logical_line):
-        yield(0, msg)
-
-
-@core.flake8ext
-def check_python3_no_iterkeys(logical_line):
-    msg = ("G331: Use six.iterkeys() or dict.keys() instead of "
-           "dict.iterkeys().")
-    if re.search(r".*\.iterkeys\(\)", logical_line):
-        yield(0, msg)
-
-
-@core.flake8ext
-def check_python3_no_itervalues(logical_line):
-    msg = ("G332: Use six.itervalues() or dict.values instead of "
-           "dict.itervalues().")
-    if re.search(r".*\.itervalues\(\)", logical_line):
-        yield(0, msg)
+    msg = ("G330: LOG.warn is deprecated, please use LOG.warning!")
+    if "LOG.warn(" in logical_line:
+        yield (0, msg)
