@@ -53,8 +53,8 @@ _api_opts = [
         glance.common.wsgi.bind_opts,
         glance.common.wsgi.eventlet_opts,
         glance.common.wsgi.socket_opts,
-        glance.common.wsgi.wsgi_opts,
         glance.common.wsgi.store_opts,
+        glance.common.wsgi.cli_opts,
         glance.image_cache.drivers.sqlite.sqlite_opts,
         glance.image_cache.image_cache_opts,
         glance.notifier.notifier_opts,
@@ -91,6 +91,8 @@ _image_import_opts = [
      glance.async_.flows.api_image_import.api_import_opts),
     ('import_filtering_opts',
      glance.async_.flows._internal_plugins.import_filtering_opts),
+    ('glance_download_opts',
+     glance.async_.flows.api_image_import.glance_download_opts)
 ]
 
 
