@@ -15,6 +15,7 @@
 
 """Functional test case that tests logging output"""
 
+import http.client as http
 import os
 import stat
 
@@ -35,20 +36,15 @@ class TestLogging(functional.FunctionalTest):
         self.start_servers()
 
         # The default functional test case has both debug on. Let's verify
-        # that debug statements appear in both the API and registry logs.
+        # that debug statements appear in the API logs.
 
         self.assertTrue(os.path.exists(self.api_server.log_file))
 
-        api_log_out = open(self.api_server.log_file, 'r').read()
+        with open(self.api_server.log_file, 'r') as f:
+            api_log_out = f.read()
 
         self.assertIn('DEBUG glance', api_log_out)
 
-        self.assertTrue(os.path.exists(self.registry_server.log_file))
-
-        registry_log_out = open(self.registry_server.log_file, 'r').read()
-
-        self.assertIn('DEBUG glance', registry_log_out)
-
         self.stop_servers()
 
     def test_no_debug(self):
@@ -60,16 +56,10 @@ class TestLogging(functional.FunctionalTest):
 
         self.assertTrue(os.path.exists(self.api_server.log_file))
 
-        api_log_out = open(self.api_server.log_file, 'r').read()
+        with open(self.api_server.log_file, 'r') as f:
+            api_log_out = f.read()
 
         self.assertNotIn('DEBUG glance', api_log_out)
-
-        self.assertTrue(os.path.exists(self.registry_server.log_file))
-
-        registry_log_out = open(self.registry_server.log_file, 'r').read()
-
-        self.assertNotIn('DEBUG glance', registry_log_out)
-
         self.stop_servers()
 
     def assertNotEmptyFile(self, path):
@@ -80,6 +70,12 @@ class TestLogging(functional.FunctionalTest):
         """
         Test that we notice when our log file has been rotated
         """
+
+        # Moving in-use files is not supported on Windows.
+        # The log handler itself may be configured to rotate files.
+        if os.name == 'nt':
+            raise self.skipException("Unsupported platform.")
+
         self.cleanup()
         self.start_servers()
 
@@ -89,7 +85,7 @@ class TestLogging(functional.FunctionalTest):
 
         path = "http://%s:%d/" % ("127.0.0.1", self.api_port)
         response, content = httplib2.Http().request(path, 'GET')
-        self.assertEqual(300, response.status)
+        self.assertEqual(http.MULTIPLE_CHOICES, response.status)
 
         self.assertNotEmptyFile(self.api_server.log_file)
 
