@@ -14,6 +14,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import io
 import tempfile
 from unittest import mock
 
@@ -21,7 +22,6 @@ import glance_store as store
 from glance_store._drivers import cinder
 from oslo_config import cfg
 from oslo_log import log as logging
-import six
 import webob
 
 from glance.common import exception
@@ -30,7 +30,6 @@ from glance.common import utils
 from glance.tests.unit import base
 from glance.tests import utils as test_utils
 
-
 CONF = cfg.CONF
 
 
@@ -276,14 +275,14 @@ class TestUtils(test_utils.BaseTestCase):
         """Ensure limiting reader class accesses all bytes of file"""
         BYTES = 1024
         bytes_read = 0
-        data = six.StringIO("*" * BYTES)
+        data = io.StringIO("*" * BYTES)
         for chunk in utils.LimitingReader(data, BYTES):
             bytes_read += len(chunk)
 
         self.assertEqual(BYTES, bytes_read)
 
         bytes_read = 0
-        data = six.StringIO("*" * BYTES)
+        data = io.StringIO("*" * BYTES)
         reader = utils.LimitingReader(data, BYTES)
         byte = reader.read(1)
         while len(byte) != 0:
@@ -298,7 +297,7 @@ class TestUtils(test_utils.BaseTestCase):
 
         def _consume_all_iter():
             bytes_read = 0
-            data = six.StringIO("*" * BYTES)
+            data = io.StringIO("*" * BYTES)
             for chunk in utils.LimitingReader(data, BYTES - 1):
                 bytes_read += len(chunk)
 
@@ -306,7 +305,7 @@ class TestUtils(test_utils.BaseTestCase):
 
         def _consume_all_read():
             bytes_read = 0
-            data = six.StringIO("*" * BYTES)
+            data = io.StringIO("*" * BYTES)
             reader = utils.LimitingReader(data, BYTES - 1)
             byte = reader.read(1)
             while len(byte) != 0:
@@ -349,8 +348,8 @@ class TestUtils(test_utils.BaseTestCase):
         actual = utils.image_meta_to_http_headers(image_meta)
         actual_test2 = utils.image_meta_to_http_headers(
             image_meta_properties)
-        self.assertEqual({'x-image-meta-x-image-meta-size': u'test'}, actual)
-        self.assertEqual({'x-image-meta-property-test': u'test'},
+        self.assertEqual({'x-image-meta-x-image-meta-size': 'test'}, actual)
+        self.assertEqual({'x-image-meta-property-test': 'test'},
                          actual_test2)
 
     def test_create_mashup_dict_with_different_core_custom_properties(self):
@@ -436,7 +435,7 @@ class TestUtils(test_utils.BaseTestCase):
     def test_valid_hostname_fail(self):
         invalid_inputs = ['localhost.localdomain',
                           '192.168.0.1',
-                          u'\u2603',
+                          '\u2603',
                           'glance02.stack42.local']
 
         for input_str in invalid_inputs:
@@ -455,7 +454,7 @@ class TestUtils(test_utils.BaseTestCase):
         invalid_inputs = ['localhost',
                           '192.168.0.1',
                           '999.88.77.6',
-                          u'\u2603.local',
+                          '\u2603.local',
                           'glance02.stack42']
 
         for input_str in invalid_inputs:
@@ -490,8 +489,8 @@ class TestUtils(test_utils.BaseTestCase):
                          '172.17.17.1:99999',
                          '290.12.52.80:5673',
                          'absurd inputs happen',
-                         u'\u2601',
-                         u'\u2603:8080',
+                         '\u2601',
+                         '\u2603:8080',
                          'fe80::1',
                          '[fe80::2]',
                          '<fe80::3>:5673',
