@@ -15,30 +15,38 @@
 
 from contextlib import contextmanager
 import datetime
-import hashlib
+import io
 import os
+import tempfile
 import time
+from unittest import mock
 
 import fixtures
+import glance_store as store
+from oslo_config import cfg
+from oslo_utils import secretutils
 from oslo_utils import units
-import six
-# NOTE(jokke): simplified transition to py3, behaves like py2 xrange
-from six.moves import range
 
+from glance import async_
 from glance.common import exception
+from glance import context
+from glance import gateway as glance_gateway
 from glance import image_cache
+from glance.image_cache import prefetcher
+from glance.tests.unit import utils as unit_test_utils
 from glance.tests import utils as test_utils
 from glance.tests.utils import skip_if_disabled
 from glance.tests.utils import xattr_writes_supported
 
 FIXTURE_LENGTH = 1024
 FIXTURE_DATA = b'*' * FIXTURE_LENGTH
+CONF = cfg.CONF
 
 
 class ImageCacheTestCase(object):
 
     def _setup_fixture_file(self):
-        FIXTURE_FILE = six.BytesIO(FIXTURE_DATA)
+        FIXTURE_FILE = io.BytesIO(FIXTURE_DATA)
 
         self.assertFalse(self.cache.is_cached(1))
 
@@ -61,7 +69,7 @@ class ImageCacheTestCase(object):
         """
         self._setup_fixture_file()
 
-        buff = six.BytesIO()
+        buff = io.BytesIO()
         with self.cache.open_for_read(1) as cache_file:
             for chunk in cache_file:
                 buff.write(chunk)
@@ -75,7 +83,7 @@ class ImageCacheTestCase(object):
         """
         self._setup_fixture_file()
 
-        buff = six.BytesIO()
+        buff = io.BytesIO()
         with self.cache.open_for_read(1) as cache_file:
             for chunk in cache_file:
                 buff.write(chunk)
@@ -109,7 +117,7 @@ class ImageCacheTestCase(object):
             self.assertFalse(self.cache.is_cached(image_id))
 
         for image_id in (1, 2):
-            FIXTURE_FILE = six.BytesIO(FIXTURE_DATA)
+            FIXTURE_FILE = io.BytesIO(FIXTURE_DATA)
             self.assertTrue(self.cache.cache_image_file(image_id,
                                                         FIXTURE_FILE))
 
@@ -179,21 +187,21 @@ class ImageCacheTestCase(object):
         # prune. We should see only 5 images left after pruning, and the
         # images that are least recently accessed should be the ones pruned...
         for x in range(10):
-            FIXTURE_FILE = six.BytesIO(FIXTURE_DATA)
+            FIXTURE_FILE = io.BytesIO(FIXTURE_DATA)
             self.assertTrue(self.cache.cache_image_file(x, FIXTURE_FILE))
 
         self.assertEqual(10 * units.Ki, self.cache.get_cache_size())
 
         # OK, hit the images that are now cached...
         for x in range(10):
-            buff = six.BytesIO()
+            buff = io.BytesIO()
             with self.cache.open_for_read(x) as cache_file:
                 for chunk in cache_file:
                     buff.write(chunk)
 
         # Add a new image to cache.
         # This is specifically to test the bug: 1438564
-        FIXTURE_FILE = six.BytesIO(FIXTURE_DATA)
+        FIXTURE_FILE = io.BytesIO(FIXTURE_DATA)
         self.assertTrue(self.cache.cache_image_file(99, FIXTURE_FILE))
 
         self.cache.prune()
@@ -221,13 +229,13 @@ class ImageCacheTestCase(object):
         """
         self.assertEqual(0, self.cache.get_cache_size())
 
-        FIXTURE_FILE = six.BytesIO(FIXTURE_DATA)
+        FIXTURE_FILE = io.BytesIO(FIXTURE_DATA)
         self.assertTrue(self.cache.cache_image_file('xxx', FIXTURE_FILE))
 
         self.assertEqual(1024, self.cache.get_cache_size())
 
         # OK, hit the image that is now cached...
-        buff = six.BytesIO()
+        buff = io.BytesIO()
         with self.cache.open_for_read('xxx') as cache_file:
             for chunk in cache_file:
                 buff.write(chunk)
@@ -247,7 +255,7 @@ class ImageCacheTestCase(object):
         self.assertFalse(self.cache.is_cached(1))
         self.assertFalse(self.cache.is_queued(1))
 
-        FIXTURE_FILE = six.BytesIO(FIXTURE_DATA)
+        FIXTURE_FILE = io.BytesIO(FIXTURE_DATA)
 
         self.assertTrue(self.cache.queue_image(1))
 
@@ -303,8 +311,8 @@ class ImageCacheTestCase(object):
         """
         Test to see if open_for_write works in a failure case for each driver
         This case is where an exception is raised while the file is being
-        written. The image is partially filled in cache and filling wont resume
-        so verify the image is moved to invalid/ directory
+        written. The image is partially filled in cache and filling won't
+        resume so verify the image is moved to invalid/ directory
         """
         # test a case where an exception is raised while the file is open
         image_id = '1'
@@ -380,7 +388,7 @@ class ImageCacheTestCase(object):
         """
         Test to see if the caching iterator interacts properly with the driver
         in a case where the iterator is only partially consumed. In this case
-        the image is only partially filled in cache and filling wont resume.
+        the image is only partially filled in cache and filling won't resume.
         When the iterator goes out of scope the driver should have closed the
         image and moved it from incomplete/ to invalid/
         """
@@ -409,7 +417,7 @@ class ImageCacheTestCase(object):
         image = b"12345678990abcdefghijklmnop"
         image_id = 123
 
-        md5 = hashlib.md5()
+        md5 = secretutils.md5(usedforsecurity=False)
         md5.update(image)
         checksum = md5.hexdigest()
 
@@ -510,6 +518,60 @@ class TestImageCacheSqlite(test_utils.BaseTestCase,
                     image_cache_max_size=5 * units.Ki)
         self.cache = image_cache.ImageCache()
 
+    @mock.patch('glance.db.get_api')
+    def _test_prefetcher(self, mock_get_db):
+        self.config(enabled_backends={'cheap': 'file'})
+        store.register_store_opts(CONF)
+        self.config(filesystem_store_datadir='/tmp', group='cheap')
+        store.create_multi_stores(CONF)
+
+        tempf = tempfile.NamedTemporaryFile()
+        tempf.write(b'foo')
+
+        db = unit_test_utils.FakeDB(initialize=False)
+        mock_get_db.return_value = db
+
+        ctx = context.RequestContext(is_admin=True, roles=['admin'])
+        gateway = glance_gateway.Gateway()
+        image_factory = gateway.get_image_factory(ctx)
+        image_repo = gateway.get_repo(ctx)
+        fetcher = prefetcher.Prefetcher()
+
+        # Create an image with no values set and queue it
+        image = image_factory.new_image()
+        image_repo.add(image)
+        fetcher.cache.queue_image(image.image_id)
+
+        # Image is not active, so it should fail to cache, but remain queued
+        self.assertFalse(fetcher.run())
+        self.assertFalse(fetcher.cache.is_cached(image.image_id))
+        self.assertTrue(fetcher.cache.is_queued(image.image_id))
+
+        # Set the disk/container format and give it a location
+        image.disk_format = 'raw'
+        image.container_format = 'bare'
+        image.status = 'active'
+        loc = {'url': 'file://%s' % tempf.name, 'metadata': {'store': 'cheap'}}
+        with mock.patch('glance.location._check_image_location'):
+            # FIXME(danms): Why do I have to do this?
+            image.locations = [loc]
+        image_repo.save(image)
+
+        # Image is now active and has a location, so it should cache
+        self.assertTrue(fetcher.run())
+        self.assertTrue(fetcher.cache.is_cached(image.image_id))
+        self.assertFalse(fetcher.cache.is_queued(image.image_id))
+
+    @mock.patch('glance.async_._THREADPOOL_MODEL', new=None)
+    def test_prefetcher_greenthread(self):
+        async_.set_threadpool_model('eventlet')
+        self._test_prefetcher()
+
+    @mock.patch('glance.async_._THREADPOOL_MODEL', new=None)
+    def test_prefetcher_native(self):
+        async_.set_threadpool_model('native')
+        self._test_prefetcher()
+
 
 class TestImageCacheNoDep(test_utils.BaseTestCase):
 
@@ -564,3 +626,19 @@ class TestImageCacheNoDep(test_utils.BaseTestCase):
 
         caching_iter = cache.get_caching_iter('dummy_id', None, iter(data))
         self.assertEqual(data, list(caching_iter))
+
+
+class TestImagePrefetcher(test_utils.BaseTestCase):
+    def setUp(self):
+        super(TestImagePrefetcher, self).setUp()
+        self.cache_dir = self.useFixture(fixtures.TempDir()).path
+        self.config(image_cache_dir=self.cache_dir,
+                    image_cache_driver='xattr',
+                    image_cache_max_size=5 * units.Ki)
+        self.prefetcher = prefetcher.Prefetcher()
+
+    def test_fetch_image_into_cache_without_auth(self):
+        with mock.patch.object(self.prefetcher.gateway,
+                               'get_repo') as mock_get:
+            self.prefetcher.fetch_image_into_cache('fake-image-id')
+            mock_get.assert_called_once_with(mock.ANY)
