@@ -633,6 +633,9 @@ class TestTaskNotifications(utils.BaseTestCase):
             expires_at=None,
             created_at=DATETIME,
             updated_at=DATETIME,
+            image_id='fake_image_id',
+            user_id='fake_user',
+            request_id='fake_request_id',
         )
 
         self.task = Task(
@@ -645,7 +648,10 @@ class TestTaskNotifications(utils.BaseTestCase):
             updated_at=DATETIME,
             task_input=task_input,
             result='res',
-            message='blah'
+            message='blah',
+            image_id='fake_image_id',
+            user_id='fake_user',
+            request_id='fake_request_id',
         )
         self.context = glance.context.RequestContext(
             tenant=TENANT2,
@@ -694,6 +700,11 @@ class TestTaskNotifications(utils.BaseTestCase):
         )
         if 'location' in output_log['payload']:
             self.fail('Notification contained location field.')
+        # Verify newly added fields 'image_id', 'user_id' and
+        # 'request_id' are not part of notification yet
+        self.assertTrue('image_id' not in output_log['payload'])
+        self.assertTrue('user_id' not in output_log['payload'])
+        self.assertTrue('request_id' not in output_log['payload'])
 
     def test_task_create_notification_disabled(self):
         self.config(disabled_notifications=['task.create'])
@@ -724,6 +735,11 @@ class TestTaskNotifications(utils.BaseTestCase):
         )
         if 'location' in output_log['payload']:
             self.fail('Notification contained location field.')
+        # Verify newly added fields 'image_id', 'user_id' and
+        # 'request_id' are not part of notification yet
+        self.assertTrue('image_id' not in output_log['payload'])
+        self.assertTrue('user_id' not in output_log['payload'])
+        self.assertTrue('request_id' not in output_log['payload'])
 
     def test_task_delete_notification_disabled(self):
         self.config(disabled_notifications=['task.delete'])
@@ -742,6 +758,15 @@ class TestTaskNotifications(utils.BaseTestCase):
         self.assertEqual('INFO', output_log['notification_type'])
         self.assertEqual('task.run', output_log['event_type'])
         self.assertEqual(self.task.task_id, output_log['payload']['id'])
+        self.assertFalse(
+            self.task.image_id in output_log['payload']
+        )
+        self.assertFalse(
+            self.task.user_id in output_log['payload']
+        )
+        self.assertFalse(
+            self.task.request_id in output_log['payload']
+        )
 
     def test_task_run_notification_disabled(self):
         self.config(disabled_notifications=['task.run'])
@@ -760,6 +785,11 @@ class TestTaskNotifications(utils.BaseTestCase):
         self.assertEqual('INFO', output_log['notification_type'])
         self.assertEqual('task.processing', output_log['event_type'])
         self.assertEqual(self.task.task_id, output_log['payload']['id'])
+        # Verify newly added fields 'image_id', 'user_id' and
+        # 'request_id' are not part of notification yet
+        self.assertTrue('image_id' not in output_log['payload'])
+        self.assertTrue('user_id' not in output_log['payload'])
+        self.assertTrue('request_id' not in output_log['payload'])
 
     def test_task_processing_notification_disabled(self):
         self.config(disabled_notifications=['task.processing'])
@@ -776,6 +806,11 @@ class TestTaskNotifications(utils.BaseTestCase):
         self.assertEqual('INFO', output_log['notification_type'])
         self.assertEqual('task.success', output_log['event_type'])
         self.assertEqual(self.task.task_id, output_log['payload']['id'])
+        # Verify newly added fields 'image_id', 'user_id' and
+        # 'request_id' are not part of notification yet
+        self.assertTrue('image_id' not in output_log['payload'])
+        self.assertTrue('user_id' not in output_log['payload'])
+        self.assertTrue('request_id' not in output_log['payload'])
 
     def test_task_success_notification_disabled(self):
         self.config(disabled_notifications=['task.processing', 'task.success'])
@@ -792,6 +827,11 @@ class TestTaskNotifications(utils.BaseTestCase):
         self.assertEqual('INFO', output_log['notification_type'])
         self.assertEqual('task.failure', output_log['event_type'])
         self.assertEqual(self.task.task_id, output_log['payload']['id'])
+        # Verify newly added fields 'image_id', 'user_id' and
+        # 'request_id' are not part of notification yet
+        self.assertTrue('image_id' not in output_log['payload'])
+        self.assertTrue('user_id' not in output_log['payload'])
+        self.assertTrue('request_id' not in output_log['payload'])
 
     def test_task_failure_notification_disabled(self):
         self.config(disabled_notifications=['task.failure'])
