@@ -12,10 +12,11 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+from cryptography import exceptions as crypto_exception
 from cursive import exception as cursive_exception
 from cursive import signature_utils
 import glance_store
-import mock
+from unittest import mock
 
 from glance.common import exception
 import glance.location
@@ -48,8 +49,14 @@ class ImageStub(object):
         self.status = status
         self.locations = locations or []
         self.visibility = visibility
-        self.size = 1
+        self.size = None
         self.extra_properties = extra_properties or {}
+        self.os_hash_algo = None
+        self.os_hash_value = None
+        self.checksum = None
+        self.disk_format = 'raw'
+        self.container_format = 'bare'
+        self.virtual_size = 0
 
     def delete(self):
         self.status = 'deleted'
@@ -84,6 +91,105 @@ class FakeMemberRepo(object):
         self.tenants.remove(member.member_id)
 
 
+class TestStoreMultiBackends(utils.BaseTestCase):
+    def setUp(self):
+        self.store_api = unit_test_utils.FakeStoreAPI()
+        self.store_utils = unit_test_utils.FakeStoreUtils(self.store_api)
+        self.enabled_backends = {
+            "ceph1": "rbd",
+            "ceph2": "rbd"
+        }
+        super(TestStoreMultiBackends, self).setUp()
+        self.config(enabled_backends=self.enabled_backends)
+
+    @mock.patch("glance.location.signature_utils.get_verifier")
+    def test_set_data_calls_upload_to_store(self, msig):
+        context = glance.context.RequestContext(user=USER1)
+        extra_properties = {
+            'img_signature_certificate_uuid': 'UUID',
+            'img_signature_hash_method': 'METHOD',
+            'img_signature_key_type': 'TYPE',
+            'img_signature': 'VALID'
+        }
+        image_stub = ImageStub(UUID2, status='queued', locations=[],
+                               extra_properties=extra_properties)
+        image_stub.disk_format = 'iso'
+        image = glance.location.ImageProxy(image_stub, context,
+                                           self.store_api, self.store_utils)
+        with mock.patch.object(image, "_upload_to_store") as mloc:
+            image.set_data('YYYY', 4, backend='ceph1')
+            msig.assert_called_once_with(context=context,
+                                         img_signature_certificate_uuid='UUID',
+                                         img_signature_hash_method='METHOD',
+                                         img_signature='VALID',
+                                         img_signature_key_type='TYPE')
+            mloc.assert_called_once_with('YYYY', msig.return_value, 'ceph1', 4)
+
+        self.assertEqual('active', image.status)
+
+    def test_image_set_data(self):
+        store_api = mock.MagicMock()
+        store_api.add_with_multihash.return_value = (
+            "rbd://ceph1", 4, "Z", "MH", {"backend": "ceph1"})
+        context = glance.context.RequestContext(user=USER1)
+        image_stub = ImageStub(UUID2, status='queued', locations=[])
+        image = glance.location.ImageProxy(image_stub, context,
+                                           store_api, self.store_utils)
+        image.set_data('YYYY', 4, backend='ceph1')
+        self.assertEqual(4, image.size)
+
+        # NOTE(markwash): FakeStore returns image_id for location
+        self.assertEqual("rbd://ceph1", image.locations[0]['url'])
+        self.assertEqual({"backend": "ceph1"}, image.locations[0]['metadata'])
+        self.assertEqual('Z', image.checksum)
+        self.assertEqual('active', image.status)
+
+    @mock.patch('glance.location.LOG')
+    def test_image_set_data_valid_signature(self, mock_log):
+        store_api = mock.MagicMock()
+        store_api.add_with_multihash.return_value = (
+            "rbd://ceph1", 4, "Z", "MH", {"backend": "ceph1"})
+        context = glance.context.RequestContext(user=USER1)
+        extra_properties = {
+            'img_signature_certificate_uuid': 'UUID',
+            'img_signature_hash_method': 'METHOD',
+            'img_signature_key_type': 'TYPE',
+            'img_signature': 'VALID'
+        }
+        image_stub = ImageStub(UUID2, status='queued',
+                               extra_properties=extra_properties)
+        self.mock_object(signature_utils, 'get_verifier',
+                         unit_test_utils.fake_get_verifier)
+        image = glance.location.ImageProxy(image_stub, context,
+                                           store_api, self.store_utils)
+        image.set_data('YYYY', 4, backend='ceph1')
+        self.assertEqual('active', image.status)
+        call = mock.call('Successfully verified signature for image %s',
+                         UUID2)
+        mock_log.info.assert_has_calls([call])
+
+    @mock.patch("glance.location.signature_utils.get_verifier")
+    def test_image_set_data_invalid_signature(self, msig):
+        msig.return_value.verify.side_effect = \
+            crypto_exception.InvalidSignature
+        store_api = mock.MagicMock()
+        store_api.add_with_multihash.return_value = (
+            "rbd://ceph1", 4, "Z", "MH", {"backend": "ceph1"})
+        context = glance.context.RequestContext(user=USER1)
+        extra_properties = {
+            'img_signature_certificate_uuid': 'UUID',
+            'img_signature_hash_method': 'METHOD',
+            'img_signature_key_type': 'TYPE',
+            'img_signature': 'INVALID'
+        }
+        image_stub = ImageStub(UUID2, status='queued',
+                               extra_properties=extra_properties)
+        image = glance.location.ImageProxy(image_stub, context,
+                                           store_api, self.store_utils)
+        self.assertRaises(cursive_exception.SignatureVerificationError,
+                          image.set_data, 'YYYY', 4, backend='ceph1')
+
+
 class TestStoreImage(utils.BaseTestCase):
     def setUp(self):
         locations = [{'url': '%s/%s' % (BASE_URI, UUID1),
@@ -124,16 +230,21 @@ class TestStoreImage(utils.BaseTestCase):
         context = glance.context.RequestContext(user=USER1)
         (image2, image_stub2) = self._add_image(context, UUID2, 'ZZZ', 3)
         location_data = image2.locations[0]
-        image1.locations.append(location_data)
+
+        with mock.patch("glance.location.store") as mock_store:
+            mock_store.get_size_from_uri_and_backend.return_value = 3
+            image1.locations.append(location_data)
+
         self.assertEqual(2, len(image1.locations))
         self.assertEqual(UUID2, location_data['url'])
 
-        self.stubs.Set(unit_test_utils.FakeStoreAPI, 'get_from_backend',
-                       fake_get_from_backend)
+        self.mock_object(unit_test_utils.FakeStoreAPI, 'get_from_backend',
+                         fake_get_from_backend)
         # This time, image1.get_data() returns the data wrapped in a
-        # LimitingReader|CooperativeReader pipeline, so peeking under
-        # the hood of those objects to get at the underlying string.
-        self.assertEqual('ZZZ', image1.get_data().data.fd)
+        # LimitingReader|CooperativeReader|InfoWrapper pipeline, so
+        # peeking under the hood of those objects to get at the
+        # underlying string.
+        self.assertEqual('ZZZ', image1.get_data().data.fd._source)
 
         image1.locations.pop(0)
         self.assertEqual(1, len(image1.locations))
@@ -142,14 +253,57 @@ class TestStoreImage(utils.BaseTestCase):
     def test_image_set_data(self):
         context = glance.context.RequestContext(user=USER1)
         image_stub = ImageStub(UUID2, status='queued', locations=[])
+        # We are going to pass an iterable data source, so use the
+        # FakeStoreAPIReader that actually reads from that data
+        store_api = unit_test_utils.FakeStoreAPIReader()
         image = glance.location.ImageProxy(image_stub, context,
-                                           self.store_api, self.store_utils)
-        image.set_data('YYYY', 4)
+                                           store_api, self.store_utils)
+        image.set_data(iter(['YYYY']), 4)
         self.assertEqual(4, image.size)
         # NOTE(markwash): FakeStore returns image_id for location
         self.assertEqual(UUID2, image.locations[0]['url'])
         self.assertEqual('Z', image.checksum)
         self.assertEqual('active', image.status)
+        self.assertEqual(4, image.virtual_size)
+
+    def test_image_set_data_inspector_no_match(self):
+        context = glance.context.RequestContext(user=USER1)
+        image_stub = ImageStub(UUID2, status='queued', locations=[])
+        image_stub.disk_format = 'qcow2'
+        # We are going to pass an iterable data source, so use the
+        # FakeStoreAPIReader that actually reads from that data
+        store_api = unit_test_utils.FakeStoreAPIReader()
+        image = glance.location.ImageProxy(image_stub, context,
+                                           store_api, self.store_utils)
+        image.set_data(iter(['YYYY']), 4)
+        self.assertEqual(4, image.size)
+        # NOTE(markwash): FakeStore returns image_id for location
+        self.assertEqual(UUID2, image.locations[0]['url'])
+        self.assertEqual('Z', image.checksum)
+        self.assertEqual('active', image.status)
+        self.assertEqual(0, image.virtual_size)
+
+    @mock.patch('glance.common.format_inspector.get_inspector')
+    def test_image_set_data_inspector_not_needed(self, mock_gi):
+        context = glance.context.RequestContext(user=USER1)
+        image_stub = ImageStub(UUID2, status='queued', locations=[])
+        image_stub.virtual_size = 123
+        image_stub.disk_format = 'qcow2'
+        # We are going to pass an iterable data source, so use the
+        # FakeStoreAPIReader that actually reads from that data
+        store_api = unit_test_utils.FakeStoreAPIReader()
+        image = glance.location.ImageProxy(image_stub, context,
+                                           store_api, self.store_utils)
+        image.set_data(iter(['YYYY']), 4)
+        self.assertEqual(4, image.size)
+        # NOTE(markwash): FakeStore returns image_id for location
+        self.assertEqual(UUID2, image.locations[0]['url'])
+        self.assertEqual('Z', image.checksum)
+        self.assertEqual('active', image.status)
+        self.assertEqual(123, image.virtual_size)
+        # If the image already had virtual_size set (i.e. we're setting
+        # a new location), we should not re-calculate the value.
+        mock_gi.assert_not_called()
 
     def test_image_set_data_location_metadata(self):
         context = glance.context.RequestContext(user=USER1)
@@ -175,6 +329,7 @@ class TestStoreImage(utils.BaseTestCase):
     def test_image_set_data_unknown_size(self):
         context = glance.context.RequestContext(user=USER1)
         image_stub = ImageStub(UUID2, status='queued', locations=[])
+        image_stub.disk_format = 'iso'
         image = glance.location.ImageProxy(image_stub, context,
                                            self.store_api, self.store_utils)
         image.set_data('YYYY', None)
@@ -200,14 +355,14 @@ class TestStoreImage(utils.BaseTestCase):
         }
         image_stub = ImageStub(UUID2, status='queued',
                                extra_properties=extra_properties)
-        self.stubs.Set(signature_utils, 'get_verifier',
-                       unit_test_utils.fake_get_verifier)
+        self.mock_object(signature_utils, 'get_verifier',
+                         unit_test_utils.fake_get_verifier)
         image = glance.location.ImageProxy(image_stub, context,
                                            self.store_api, self.store_utils)
         image.set_data('YYYY', 4)
         self.assertEqual('active', image.status)
-        mock_log.info.assert_called_once_with(
-            u'Successfully verified signature for image %s',
+        mock_log.info.assert_any_call(
+            'Successfully verified signature for image %s',
             UUID2)
 
     def test_image_set_data_invalid_signature(self):
@@ -220,8 +375,8 @@ class TestStoreImage(utils.BaseTestCase):
         }
         image_stub = ImageStub(UUID2, status='queued',
                                extra_properties=extra_properties)
-        self.stubs.Set(signature_utils, 'get_verifier',
-                       unit_test_utils.fake_get_verifier)
+        self.mock_object(signature_utils, 'get_verifier',
+                         unit_test_utils.fake_get_verifier)
         image = glance.location.ImageProxy(image_stub, context,
                                            self.store_api, self.store_utils)
         with mock.patch.object(self.store_api,
@@ -240,8 +395,8 @@ class TestStoreImage(utils.BaseTestCase):
         }
         image_stub = ImageStub(UUID2, status='queued',
                                extra_properties=extra_properties)
-        self.stubs.Set(signature_utils, 'get_verifier',
-                       unit_test_utils.fake_get_verifier)
+        self.mock_object(signature_utils, 'get_verifier',
+                         unit_test_utils.fake_get_verifier)
         image = glance.location.ImageProxy(image_stub, context,
                                            self.store_api, self.store_utils)
         image.set_data('YYYY', 4)
@@ -603,8 +758,9 @@ class TestStoreImage(utils.BaseTestCase):
 
         location2 = {'url': UUID2, 'metadata': {}}
         location3 = {'url': UUID3, 'metadata': {}}
-
-        image3.locations += [location2, location3]
+        with mock.patch("glance.location.store") as mock_store:
+            mock_store.get_size_from_uri_and_backend.return_value = 4
+            image3.locations += [location2, location3]
 
         self.assertEqual([location2, location3], image_stub3.locations)
         self.assertEqual([location2, location3], image3.locations)
@@ -633,7 +789,9 @@ class TestStoreImage(utils.BaseTestCase):
         location2 = {'url': UUID2, 'metadata': {}}
         location3 = {'url': UUID3, 'metadata': {}}
 
-        image3.locations += [location2, location3]
+        with mock.patch("glance.location.store") as mock_store:
+            mock_store.get_size_from_uri_and_backend.return_value = 4
+            image3.locations += [location2, location3]
 
         self.assertEqual(1, image_stub3.locations.index(location3))
 
@@ -660,7 +818,9 @@ class TestStoreImage(utils.BaseTestCase):
         location2 = {'url': UUID2, 'metadata': {}}
         location3 = {'url': UUID3, 'metadata': {}}
 
-        image3.locations += [location2, location3]
+        with mock.patch("glance.location.store") as mock_store:
+            mock_store.get_size_from_uri_and_backend.return_value = 4
+            image3.locations += [location2, location3]
 
         self.assertEqual(1, image_stub3.locations.index(location3))
         self.assertEqual(location2, image_stub3.locations[0])
@@ -690,7 +850,9 @@ class TestStoreImage(utils.BaseTestCase):
         location3 = {'url': UUID3, 'metadata': {}}
         location_bad = {'url': 'unknown://location', 'metadata': {}}
 
-        image3.locations += [location2, location3]
+        with mock.patch("glance.location.store") as mock_store:
+            mock_store.get_size_from_uri_and_backend.return_value = 4
+            image3.locations += [location2, location3]
 
         self.assertIn(location3, image_stub3.locations)
         self.assertNotIn(location_bad, image_stub3.locations)
@@ -718,7 +880,9 @@ class TestStoreImage(utils.BaseTestCase):
         image_stub3 = ImageStub('fake_image_id', status='queued', locations=[])
         image3 = glance.location.ImageProxy(image_stub3, context,
                                             self.store_api, self.store_utils)
-        image3.locations += [location2, location3]
+        with mock.patch("glance.location.store") as mock_store:
+            mock_store.get_size_from_uri_and_backend.return_value = 4
+            image3.locations += [location2, location3]
 
         image_stub3.locations.reverse()
 
@@ -871,25 +1035,25 @@ class TestStoreMetaDataChecker(utils.BaseTestCase):
         glance_store.check_location_metadata({})
 
     def test_unicode(self):
-        m = {'key': u'somevalue'}
+        m = {'key': 'somevalue'}
         glance_store.check_location_metadata(m)
 
     def test_unicode_list(self):
-        m = {'key': [u'somevalue', u'2']}
+        m = {'key': ['somevalue', '2']}
         glance_store.check_location_metadata(m)
 
     def test_unicode_dict(self):
-        inner = {'key1': u'somevalue', 'key2': u'somevalue'}
+        inner = {'key1': 'somevalue', 'key2': 'somevalue'}
         m = {'topkey': inner}
         glance_store.check_location_metadata(m)
 
     def test_unicode_dict_list(self):
-        inner = {'key1': u'somevalue', 'key2': u'somevalue'}
-        m = {'topkey': inner, 'list': [u'somevalue', u'2'], 'u': u'2'}
+        inner = {'key1': 'somevalue', 'key2': 'somevalue'}
+        m = {'topkey': inner, 'list': ['somevalue', '2'], 'u': '2'}
         glance_store.check_location_metadata(m)
 
     def test_nested_dict(self):
-        inner = {'key1': u'somevalue', 'key2': u'somevalue'}
+        inner = {'key1': 'somevalue', 'key2': 'somevalue'}
         inner = {'newkey': inner}
         inner = {'anotherkey': inner}
         m = {'topkey': inner}
@@ -902,13 +1066,13 @@ class TestStoreMetaDataChecker(utils.BaseTestCase):
                           m)
 
     def test_list_bad(self):
-        m = {'key1': [u'somevalue', object()]}
+        m = {'key1': ['somevalue', object()]}
         self.assertRaises(glance_store.BackendException,
                           glance_store.check_location_metadata,
                           m)
 
     def test_nested_dict_bad(self):
-        inner = {'key1': u'somevalue', 'key2': object()}
+        inner = {'key1': 'somevalue', 'key2': object()}
         inner = {'newkey': inner}
         inner = {'anotherkey': inner}
         m = {'topkey': inner}
