@@ -12,14 +12,16 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+
+import http.client as http
+import io
 from unittest import mock
 import uuid
 
 from cursive import exception as cursive_exception
 import glance_store
 from glance_store._drivers import filesystem
-import six
-from six.moves import http_client as http
+from oslo_config import cfg
 import webob
 
 import glance.api.policy
@@ -30,6 +32,9 @@ from glance.tests.unit import base
 import glance.tests.unit.utils as unit_test_utils
 import glance.tests.utils as test_utils
 
+CONF = cfg.CONF
+CONF.import_opt('public_endpoint', 'glance.api.versions')
+
 
 class Raise(object):
 
@@ -44,7 +49,7 @@ class FakeImage(object):
 
     def __init__(self, image_id=None, data=None, checksum=None, size=0,
                  virtual_size=0, locations=None, container_format='bear',
-                 disk_format='rawr', status=None):
+                 disk_format='rawr', status=None, owner=None):
         self.image_id = image_id
         self.data = data
         self.checksum = checksum
@@ -53,7 +58,9 @@ class FakeImage(object):
         self.locations = locations
         self.container_format = container_format
         self.disk_format = disk_format
+        self.owner = owner
         self._status = status
+        self.extra_properties = {}
 
     @property
     def status(self):
@@ -120,6 +127,12 @@ class TestImagesController(base.StoreClearingUnitTest):
         self.controller = glance.api.v2.image_data.ImageDataController()
         self.controller.gateway = FakeGateway(db, store, notifier, policy,
                                               self.image_repo)
+        # FIXME(abhishekk): Everything is fake in this test, so mocked the
+        # image mutable_check, Later we need to fix these tests to use
+        # some realistic data
+        patcher = mock.patch('glance.api.v2.policy.check_is_image_mutable')
+        patcher.start()
+        self.addCleanup(patcher.stop)
 
     def test_download(self):
         request = unit_test_utils.get_fake_request()
@@ -175,12 +188,34 @@ class TestImagesController(base.StoreClearingUnitTest):
 
     def test_upload(self):
         request = unit_test_utils.get_fake_request()
-        image = FakeImage('abcd')
+        image = FakeImage('abcd', owner='tenant1')
         self.image_repo.result = image
         self.controller.upload(request, unit_test_utils.UUID2, 'YYYY', 4)
         self.assertEqual('YYYY', image.data)
         self.assertEqual(4, image.size)
 
+    def test_upload_not_allowed_by_policy(self):
+        request = unit_test_utils.get_fake_request()
+        with mock.patch.object(self.controller.policy, 'enforce') as mock_enf:
+            mock_enf.side_effect = webob.exc.HTTPForbidden()
+            exc = self.assertRaises(webob.exc.HTTPNotFound,
+                                    self.controller.upload, request,
+                                    unit_test_utils.UUID1, 'YYYY', 4)
+            self.assertTrue(mock_enf.called)
+        # Make sure we did not leak details of the original Forbidden
+        # error into the NotFound returned to the client.
+        self.assertEqual('The resource could not be found.', str(exc))
+
+        # Now reject the upload_image call, but allow get_image to ensure that
+        # we properly see a Forbidden result.
+        with mock.patch.object(self.controller.policy, 'enforce') as mock_enf:
+            mock_enf.side_effect = [webob.exc.HTTPForbidden(),
+                                    lambda *a: None]
+            exc = self.assertRaises(webob.exc.HTTPForbidden,
+                                    self.controller.upload, request,
+                                    unit_test_utils.UUID1, 'YYYY', 4)
+            self.assertTrue(mock_enf.called)
+
     def test_upload_status(self):
         request = unit_test_utils.get_fake_request()
         image = FakeImage('abcd')
@@ -209,12 +244,16 @@ class TestImagesController(base.StoreClearingUnitTest):
     @mock.patch.object(glance.api.policy.Enforcer, 'enforce')
     def test_upload_image_forbidden(self, mock_enforce):
         request = unit_test_utils.get_fake_request()
-        mock_enforce.side_effect = exception.Forbidden
+        image = FakeImage('abcd', owner='tenant1')
+        self.image_repo.result = image
+        mock_enforce.side_effect = [exception.Forbidden, lambda *a: None]
         self.assertRaises(webob.exc.HTTPForbidden, self.controller.upload,
                           request, unit_test_utils.UUID2, 'YYYY', 4)
-        mock_enforce.assert_called_once_with(request.context,
-                                             "upload_image",
-                                             {})
+        expected_call = [
+            mock.call(mock.ANY, 'upload_image', mock.ANY),
+            mock.call(mock.ANY, 'get_image', mock.ANY)
+        ]
+        mock_enforce.has_calls(expected_call)
 
     def test_upload_invalid(self):
         request = unit_test_utils.get_fake_request()
@@ -241,6 +280,27 @@ class TestImagesController(base.StoreClearingUnitTest):
         self.assertEqual(3, mocked_save.call_count)
         mocked_delete.assert_called_once_with()
 
+    @mock.patch('glance.common.trust_auth.TokenRefresher')
+    def test_upload_with_token_refresh(self, mock_refresher):
+        mock_refresher.return_value = mock.MagicMock()
+        mocked_save = mock.Mock()
+        mocked_save.side_effect = [lambda *a: None,
+                                   exception.NotAuthenticated(),
+                                   lambda *a: None]
+        request = unit_test_utils.get_fake_request()
+        request.environ['keystone.token_info'] = {
+            'token': {
+                'roles': [{'name': 'member'}]
+            }
+        }
+        image = FakeImage('abcd', owner='tenant1')
+        self.image_repo.result = image
+        self.image_repo.save = mocked_save
+        self.controller.upload(request, unit_test_utils.UUID2, 'YYYY', 4)
+        self.assertEqual('YYYY', image.data)
+        self.assertEqual(4, image.size)
+        self.assertEqual(3, mocked_save.call_count)
+
     def test_upload_non_existent_image_during_save_initiates_deletion(self):
         def fake_save_not_found(self, from_state=None):
             raise exception.ImageNotFound()
@@ -320,7 +380,7 @@ class TestImagesController(base.StoreClearingUnitTest):
         self.image_repo.result = image
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.upload,
                           request, unit_test_utils.UUID1, 'YYYY', 4)
-        self.assertEqual('killed', self.image_repo.saved_image.status)
+        self.assertEqual('queued', self.image_repo.saved_image.status)
 
     def test_image_size_limit_exceeded(self):
         request = unit_test_utils.get_fake_request()
@@ -450,10 +510,11 @@ class TestImagesController(base.StoreClearingUnitTest):
         request = unit_test_utils.get_fake_request()
         image = FakeImage(image_id=image_id)
         self.image_repo.result = image
-        with mock.patch.object(filesystem.Store, 'add'):
+        with mock.patch.object(filesystem.Store, 'add') as mock_add:
+            mock_add.return_value = ('foo://bar', 4, 'ident', {})
             self.controller.stage(request, image_id, 'YYYY', 4)
         self.assertEqual('uploading', image.status)
-        self.assertEqual(0, image.size)
+        self.assertEqual(4, image.size)
 
     def test_image_already_on_staging(self):
         image_id = str(uuid.uuid4())
@@ -461,10 +522,11 @@ class TestImagesController(base.StoreClearingUnitTest):
         image = FakeImage(image_id=image_id)
         self.image_repo.result = image
         with mock.patch.object(filesystem.Store, 'add') as mock_store_add:
+            mock_store_add.return_value = ('foo://bar', 4, 'ident', {})
             self.controller.stage(request, image_id, 'YYYY', 4)
             self.assertEqual('uploading', image.status)
             mock_store_add.side_effect = glance_store.Duplicate()
-            self.assertEqual(0, image.size)
+            self.assertEqual(4, image.size)
             self.assertRaises(webob.exc.HTTPConflict, self.controller.stage,
                               request, image_id, 'YYYY', 4)
 
@@ -544,15 +606,61 @@ class TestImagesController(base.StoreClearingUnitTest):
         request = unit_test_utils.get_fake_request()
         image = FakeImage(image_id=image_id)
         self.image_repo.result = image
-        self.controller.stage(request, image_id, 'YYYY', 4)
+        with mock.patch.object(filesystem.Store, 'add') as mock_add:
+            mock_add.return_value = ('foo://bar', 4, 'ident', {})
+            self.controller.stage(request, image_id, 'YYYY', 4)
         self.assertEqual('uploading', image.status)
-        self.assertEqual(0, image.size)
+        self.assertEqual(4, image.size)
         # try staging again
         mock_store_add.side_effect = exception.InvalidImageStatusTransition(
             cur_status='uploading', new_status='uploading')
         self.assertRaises(webob.exc.HTTPConflict, self.controller.stage,
                           request, image_id, 'YYYY', 4)
 
+    def _test_image_stage_records_host(self, expected_url):
+        image_id = str(uuid.uuid4())
+        request = unit_test_utils.get_fake_request()
+        image = FakeImage(image_id=image_id)
+        self.image_repo.result = image
+        with mock.patch.object(filesystem.Store, 'add') as mock_add:
+            mock_add.return_value = ('foo://bar', 4, 'ident', {})
+            self.controller.stage(request, image_id, 'YYYY', 4)
+        if expected_url is None:
+            self.assertNotIn('os_glance_stage_host', image.extra_properties)
+        else:
+            self.assertEqual(expected_url,
+                             image.extra_properties['os_glance_stage_host'])
+
+    def test_image_stage_records_host_unset(self):
+        # Make sure we do not set a null staging host, if we are not configured
+        # to support worker-to-worker communication.
+        self._test_image_stage_records_host(None)
+
+    def test_image_stage_records_host_public_endpoint(self):
+        # Make sure we fall back to public_endpoint
+        self.config(public_endpoint='http://lb.example.com')
+        self._test_image_stage_records_host('http://lb.example.com')
+
+    def test_image_stage_records_host_self_url(self):
+        # Make sure worker_self_reference_url takes precedence
+        self.config(worker_self_reference_url='http://worker1.example.com')
+        self._test_image_stage_records_host('http://worker1.example.com')
+
+    def test_image_stage_fail_does_not_set_host(self):
+        # Make sure that if the store.add() fails, we do not claim to have the
+        # image staged.
+        self.config(public_endpoint='http://worker1.example.com')
+        image_id = str(uuid.uuid4())
+        request = unit_test_utils.get_fake_request()
+        image = FakeImage(image_id=image_id)
+        self.image_repo.result = image
+        exc_cls = glance_store.exceptions.StorageFull
+        with mock.patch.object(filesystem.Store, 'add', side_effect=exc_cls):
+            self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
+                              self.controller.stage,
+                              request, image_id, 'YYYY', 4)
+        self.assertNotIn('os_glance_stage_host', image.extra_properties)
+
 
 class TestImageDataDeserializer(test_utils.BaseTestCase):
 
@@ -576,7 +684,7 @@ class TestImageDataDeserializer(test_utils.BaseTestCase):
         request.headers['Content-Type'] = 'application/octet-stream'
         # If we use body_file, webob assumes we want to do a chunked upload,
         # ignoring the Content-Length header
-        request.body_file = six.StringIO('YYY')
+        request.body_file = io.StringIO('YYY')
         output = self.deserializer.upload(request)
         data = output.pop('data')
         self.assertEqual('YYY', data.read())
@@ -586,7 +694,7 @@ class TestImageDataDeserializer(test_utils.BaseTestCase):
     def test_upload_chunked_with_content_length(self):
         request = unit_test_utils.get_fake_request()
         request.headers['Content-Type'] = 'application/octet-stream'
-        request.body_file = six.BytesIO(b'YYY')
+        request.body_file = io.BytesIO(b'YYY')
         # The deserializer shouldn't care if the Content-Length is
         # set when the user is attempting to send chunked data.
         request.headers['Content-Length'] = 3
@@ -627,7 +735,7 @@ class TestImageDataDeserializer(test_utils.BaseTestCase):
         req = unit_test_utils.get_fake_request()
         req.headers['Content-Type'] = 'application/octet-stream'
         req.headers['Content-Length'] = 4
-        req.body_file = six.BytesIO(b'YYYY')
+        req.body_file = io.BytesIO(b'YYYY')
         output = self.deserializer.stage(req)
         data = output.pop('data')
         self.assertEqual(b'YYYY', data.read())
@@ -744,7 +852,7 @@ class TestImageDataSerializer(test_utils.BaseTestCase):
                           response, image)
 
     def test_download_failure_with_valid_range(self):
-        with mock.patch.object(glance.api.policy.ImageProxy,
+        with mock.patch.object(glance.domain.proxy.Image,
                                'get_data') as mock_get_data:
             mock_get_data.side_effect = glance_store.NotFound(image="image")
         request = wsgi.Request.blank('/')
@@ -806,7 +914,7 @@ class TestImageDataSerializer(test_utils.BaseTestCase):
         download_failures_ContentRange('bytes 4-8/3')
 
     def test_download_failure_with_valid_content_range(self):
-        with mock.patch.object(glance.api.policy.ImageProxy,
+        with mock.patch.object(glance.domain.proxy.Image,
                                'get_data') as mock_get_data:
             mock_get_data.side_effect = glance_store.NotFound(image="image")
         request = wsgi.Request.blank('/')
@@ -841,7 +949,7 @@ class TestImageDataSerializer(test_utils.BaseTestCase):
         def get_data(*args, **kwargs):
             raise exception.Forbidden()
 
-        self.mock_object(glance.api.policy.ImageProxy,
+        self.mock_object(glance.domain.proxy.Image,
                          'get_data',
                          get_data)
         request = wsgi.Request.blank('/')
@@ -860,7 +968,7 @@ class TestImageDataSerializer(test_utils.BaseTestCase):
         Make sure that serializer returns 204 no content error in case of
         image data is not available at specified location.
         """
-        with mock.patch.object(glance.api.policy.ImageProxy,
+        with mock.patch.object(glance.domain.proxy.Image,
                                'get_data') as mock_get_data:
             mock_get_data.side_effect = glance_store.NotFound(image="image")
 
@@ -875,7 +983,7 @@ class TestImageDataSerializer(test_utils.BaseTestCase):
 
     def test_download_service_unavailable(self):
         """Test image download returns HTTPServiceUnavailable."""
-        with mock.patch.object(glance.api.policy.ImageProxy,
+        with mock.patch.object(glance.domain.proxy.Image,
                                'get_data') as mock_get_data:
             mock_get_data.side_effect = glance_store.RemoteServiceUnavailable()
 
@@ -894,7 +1002,7 @@ class TestImageDataSerializer(test_utils.BaseTestCase):
         Make sure that serializer returns 400 bad request error in case of
         getting images from this store is not supported at specified location.
         """
-        with mock.patch.object(glance.api.policy.ImageProxy,
+        with mock.patch.object(glance.domain.proxy.Image,
                                'get_data') as mock_get_data:
             mock_get_data.side_effect = glance_store.StoreGetNotSupported()
 
@@ -914,7 +1022,7 @@ class TestImageDataSerializer(test_utils.BaseTestCase):
         getting randomly images from this store is not supported at
         specified location.
         """
-        with mock.patch.object(glance.api.policy.ImageProxy,
+        with mock.patch.object(glance.domain.proxy.Image,
                                'get_data') as m_get_data:
             err = glance_store.StoreRandomGetNotSupported(offset=0,
                                                           chunk_size=0)
@@ -962,6 +1070,12 @@ class TestMultiBackendImagesController(base.MultiStoreClearingUnitTest):
         self.controller = glance.api.v2.image_data.ImageDataController()
         self.controller.gateway = FakeGateway(db, store, notifier, policy,
                                               self.image_repo)
+        # FIXME(abhishekk): Everything is fake in this test, so mocked the
+        # image muntable_check, Later we need to fix these tests to use
+        # some realistic data
+        patcher = mock.patch('glance.api.v2.policy.check_is_image_mutable')
+        patcher.start()
+        self.addCleanup(patcher.stop)
 
     def test_upload(self):
         request = unit_test_utils.get_fake_request()
