@@ -17,6 +17,9 @@
 
 import errno
 import functools
+import http.client
+import http.server
+import io
 import os
 import shlex
 import shutil
@@ -35,12 +38,10 @@ from oslo_log.fixture import logging_error as log_fixture
 from oslo_log import log
 from oslo_utils import timeutils
 from oslo_utils import units
-import six
-from six.moves import BaseHTTPServer
-from six.moves import http_client as http
 import testtools
 import webob
 
+from glance.api.v2 import cached_images
 from glance.common import config
 from glance.common import exception
 from glance.common import property_utils
@@ -94,8 +95,12 @@ class BaseTestCase(testtools.TestCase):
         self.useFixture(log_fixture.get_logging_handle_error_fixture())
         self.useFixture(glance_fixtures.StandardLogging())
 
+        if cached_images.WORKER:
+            cached_images.WORKER.terminate()
+            cached_images.WORKER = None
+
     def set_policy(self):
-        conf_file = "policy.json"
+        conf_file = "policy.yaml"
         self.policy_file = self._copy_data_file(conf_file, self.conf_dir)
         self.config(policy_file=self.policy_file, group='oslo_policy')
 
@@ -142,7 +147,7 @@ class BaseTestCase(testtools.TestCase):
         self._config_fixture.config(**kw)
 
     def mock_object(self, obj, attr_name, *args, **kwargs):
-        """"Use python mock to mock an object attirbute
+        """"Use python mock to mock an object attribute
 
         Mocks the specified objects attribute with the given value.
         Automatically performs 'addCleanup' for the mock.
@@ -269,10 +274,7 @@ def fork_exec(cmd,
                         os.dup2(fptr.fileno(), desc)
                     except OSError:
                         pass
-        if pass_fds and hasattr(os, 'set_inheritable'):
-            # os.set_inheritable() is only available and needed
-            # since Python 3.4. On Python 3.3 and older, file descriptors are
-            # inheritable by default.
+        if pass_fds:
             for fd in pass_fds:
                 os.set_inheritable(fd, True)
 
@@ -317,16 +319,16 @@ def wait_for_fork(pid,
                     pass
                 elif not nice_timer.expired():
                     # Politely ask the process to GTFO
-                    LOG.warning('Killing child %i with SIGTERM' % pid)
+                    LOG.warning('Killing child %i with SIGTERM', pid)
                     os.kill(pid, signal.SIGTERM)
                 else:
                     # No more Mr. Nice Guy
-                    LOG.warning('Killing child %i with SIGKILL' % pid)
+                    LOG.warning('Killing child %i with SIGKILL', pid)
                     os.kill(pid, signal.SIGKILL)
                     expected_exitcode = signal.SIGKILL
                 time.sleep(1)
                 continue
-            LOG.info('waitpid(%i) returned %i,%i' % (pid, cpid, rc))
+            LOG.info('waitpid(%i) returned %i,%i', pid, cpid, rc)
             if rc != expected_exitcode:
                 raise RuntimeError('The exit code %d is not %d'
                                    % (rc, expected_exitcode))
@@ -335,7 +337,7 @@ def wait_for_fork(pid,
             # Nothing to wait for
             return 0
         except Exception as e:
-            LOG.error('Got wait error: %s' % e)
+            LOG.error('Got wait error: %s', e)
             if raise_error:
                 raise
 
@@ -546,21 +548,21 @@ def minimal_add_command(port, name, suffix='', public=True):
 
 def start_http_server(image_id, image_data):
     def _get_http_handler_class(fixture):
-        class StaticHTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
+        class StaticHTTPRequestHandler(http.server.BaseHTTPRequestHandler):
             def do_GET(self):
-                self.send_response(http.OK)
+                self.send_response(http.client.OK)
                 self.send_header('Content-Length', str(len(fixture)))
                 self.end_headers()
-                self.wfile.write(six.b(fixture))
+                self.wfile.write(fixture.encode('latin-1'))
                 return
 
             def do_HEAD(self):
                 # reserve non_existing_image_path for the cases where we expect
                 # 404 from the server
                 if 'non_existing_image_path' in self.path:
-                    self.send_response(http.NOT_FOUND)
+                    self.send_response(http.client.NOT_FOUND)
                 else:
-                    self.send_response(http.OK)
+                    self.send_response(http.client.OK)
                 self.send_header('Content-Length', str(len(fixture)))
                 self.end_headers()
                 return
@@ -574,7 +576,7 @@ def start_http_server(image_id, image_data):
 
     server_address = ('127.0.0.1', 0)
     handler_class = _get_http_handler_class(image_data)
-    httpd = BaseHTTPServer.HTTPServer(server_address, handler_class)
+    httpd = http.server.HTTPServer(server_address, handler_class)
     port = httpd.socket.getsockname()[1]
 
     thread = threading.Thread(target=httpd.serve_forever)
@@ -616,10 +618,10 @@ class FakeAuthMiddleware(wsgi.Middleware):
 
 
 class FakeHTTPResponse(object):
-    def __init__(self, status=http.OK, headers=None, data=None,
+    def __init__(self, status=http.client.OK, headers=None, data=None,
                  *args, **kwargs):
         data = data or b'I am a teapot, short and stout\n'
-        self.data = six.BytesIO(data)
+        self.data = io.BytesIO(data)
         self.read = self.data.read
         self.status = status
         self.headers = headers or {'content-length': len(data)}
@@ -700,18 +702,12 @@ def db_sync(version='heads', engine=None):
     alembic_command.upgrade(alembic_config, version)
 
 
-def is_sqlite_version_prior_to(major, minor):
-    import sqlite3
-    tup = sqlite3.sqlite_version_info
-    return tup[0] < major or (tup[0] == major and tup[1] < minor)
-
-
 def start_standalone_http_server():
     def _get_http_handler_class():
-        class StaticHTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
+        class StaticHTTPRequestHandler(http.server.BaseHTTPRequestHandler):
             def do_GET(self):
                 data = b"Hello World!!!"
-                self.send_response(http.OK)
+                self.send_response(http.client.OK)
                 self.send_header('Content-Length', str(len(data)))
                 self.end_headers()
                 self.wfile.write(data)
@@ -721,7 +717,7 @@ def start_standalone_http_server():
 
     server_address = ('127.0.0.1', 0)
     handler_class = _get_http_handler_class()
-    httpd = BaseHTTPServer.HTTPServer(server_address, handler_class)
+    httpd = http.server.HTTPServer(server_address, handler_class)
     port = httpd.socket.getsockname()[1]
 
     thread = threading.Thread(target=httpd.serve_forever)
